@@ -1,51 +1,74 @@
 """
-Put and Get TiddlyWeb things to and from some store.
+Store TiddlyWeb entities to the configured StorageInterface.
+
+This module provides the facade for accessing one of many possible
+modules which provide storage for entities. It provides a general
+interface to get, put, delete or list entities. StorageInterface
+implementations do the actual interaction with the the storage medium.
 """
 
+from copy import deepcopy
 
-class StoreMethodNotImplemented(IOError):
+
+class StoreError(IOError):
+    """
+    Base Exception for Store Exceptions.
+    """
+    def __str__(self):
+        # self.args may or may not be a string, and when that
+        # is the case is proving rather difficult to tell between
+        # minor and micro versions of Python. woot!
+        # So here we do some extra work.
+        message = []
+        for arg in self.args:
+            if isinstance(arg, basestring):
+                message.append(arg)
+        return ' '.join(message)
+
+
+class StoreMethodNotImplemented(StoreError):
     """
     A StorageInterface does not implement this method.
     """
     pass
 
 
-class NoBagError(IOError):
+class NoBagError(StoreError):
     """
     No Bag was found.
     """
     pass
 
 
-class NoRecipeError(IOError):
+class NoRecipeError(StoreError):
     """
     No Recipe was found.
     """
     pass
 
 
-class NoTiddlerError(IOError):
+class NoTiddlerError(StoreError):
     """
     No Tiddler was found.
     """
     pass
 
 
-class NoUserError(IOError):
+class NoUserError(StoreError):
     """
     No User was found.
     """
     pass
 
 
-class StoreLockError(IOError):
+class StoreLockError(StoreError):
     """
     This process was unable to get a lock on the store.
     """
     pass
 
 
-class StoreEncodingError(IOError):
+class StoreEncodingError(StoreError):
     """
     Something about an entity made it impossible to be
     encoded to the form require by the store.
@@ -53,6 +76,19 @@ class StoreEncodingError(IOError):
     pass
 
 
+EMPTY_HOOKS = {
+        'put': [],
+        'delete': [],
+        'get': [],
+        }
+HOOKS = {
+        'recipe': deepcopy(EMPTY_HOOKS),
+        'bag': deepcopy(EMPTY_HOOKS),
+        'tiddler': deepcopy(EMPTY_HOOKS),
+        'user': deepcopy(EMPTY_HOOKS),
+        }
+
+
 class Store(object):
     """
     Provide a facade around implementations of StorageInterface
@@ -89,7 +125,9 @@ class Store(object):
         Delete a known object.
         """
         func = self._figure_function('delete', thing)
-        return func(thing)
+        result = func(thing)
+        self._do_hook('delete', thing)
+        return result
 
     def get(self, thing):
         """
@@ -98,6 +136,7 @@ class Store(object):
         func = self._figure_function('get', thing)
         thing = func(thing)
         thing.store = self
+        self._do_hook('get', thing)
         return thing
 
     def put(self, thing):
@@ -108,7 +147,9 @@ class Store(object):
         wrong type?
         """
         func = self._figure_function('put', thing)
-        return func(thing)
+        result = func(thing)
+        self._do_hook('put', thing)
+        return result
 
     def _figure_function(self, activity, storable):
         """
@@ -129,6 +170,13 @@ class Store(object):
         list_func = getattr(self.storage, 'list_bags')
         return list_func()
 
+    def list_bag_tiddlers(self, bag):
+        """
+        List all the tiddlers in the bag.
+        """
+        list_func = getattr(self.storage, 'list_bag_tiddlers')
+        return list_func(bag)
+
     def list_recipes(self):
         """
         List all the available recipes in the system.
@@ -157,3 +205,24 @@ class Store(object):
         """
         list_func = getattr(self.storage, 'search')
         return list_func(search_query)
+
+    def _do_hook(self, method, thing):
+        """
+        Call the hook in HOOKS identified by method on thing.
+        """
+        hooked_class = thing.__class__.__name__.lower()
+        hooks = _get_hooks(method, hooked_class)
+        for hook in hooks:
+            hook(self, thing)
+
+
+def _get_hooks(method, name):
+    """
+    Look in HOOKS for the list of functions to run
+    for the class of things named by name when store
+    method method is called.
+    """
+    try:
+        return HOOKS[name][method]
+    except KeyError:
+        return []
